<?php
/***********************************************************************

  Copyright (C) 2002, 2003  Rickard Andersson (rickard@punbb.org)

  This file is part of PunBB.

  PunBB is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published
  by the Free Software Foundation; either version 2 of the License,
  or (at your option) any later version.

  PunBB is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
  MA  02111-1307  USA

************************************************************************/


// This script updates the forum database from versions 1.0 and 1.0.1 to 1.1.
// Copy this file to the forum root directory and run it. Then remove it from
// the root directory.


$update_from = array('1.0', '1.0.1');
$update_to = '1.1';


$pun_root = './';
@include $pun_root.'config.php';

// If PUN isn't defined, config.php is missing or corrupt or we are outside the root directory
if (!defined('PUN'))
	exit('This file must be run from the forum root directory.');

// Enable debug mode
define('PUN_DEBUG', 1);

// Disable error reporting for uninitialized variables
error_reporting(E_ERROR | E_WARNING | E_PARSE);

// Turn off magic_quotes_runtime
set_magic_quotes_runtime(0);

// Turn off PHP time limit
@set_time_limit(0);


// Load the functions script
require $pun_root.'include/functions.php';


// Load DB abstraction layer and try to connect
require $pun_root.'include/dblayer/common_db.php';


// Make sure we are running MySQL
if ($db_type != 'mysql')
	error('This update script can only update PunBB forums run on MySQL. This forum doesn\'t appear to be running MySQL.');	


// Check current version
$result = $db->query('SELECT cur_version FROM '.$db->prefix.'options');
if (!$result || !in_array($db->result($result, 0), $update_from))
	error('Version mismatch. This script updates version '.implode(' and ', $update_from).' to version '.$update_to.'. The database \''.$db_name.'\' doesn\'t seem to be running a supported version.', __FILE__, __LINE__);


// Get the forum options and permissions (one last time!)
$result = $db->query('SELECT * FROM '.$db->prefix.'options, '.$db->prefix.'permissions') or error('Unable to fetch forum options and permissions', __FILE__, __LINE__, $db->error());
$optperm = $db->fetch_assoc($result);

// The first 48 elements should be options and the rest permissions
list($options, $permissions) = array_chunk($optperm, 48, true);



// Create tables config and subscription
$sql = 'CREATE TABLE '.$db_prefix."config (
		conf_name VARCHAR(255) NOT NULL,
		conf_value TEXT DEFAULT NULL,
		PRIMARY KEY (conf_name)
		) TYPE=MyISAM;";

$db->query($sql) or error('Unable to create table \''.$db_prefix.'config\'. Please check your settings and try again.', __FILE__, __LINE__, $db->error());


$sql = 'CREATE TABLE '.$db_prefix."subscriptions (
		user_id INT(10) UNSIGNED NOT NULL DEFAULT '0',
		topic_id INT(10) UNSIGNED NOT NULL DEFAULT '0'
		) TYPE=MyISAM;";

$db->query($sql) or error('Unable to create table \''.$db_prefix.'subscriptions\'. Please check your settings and try again.', __FILE__, __LINE__, $db->error());



// Update DB structure
$db->query('DROP TABLE '.$db->prefix.'options, '.$db->prefix.'permissions') or error('Unable to alter DB structure.', __FILE__, __LINE__, $db->error());
$db->query('ALTER TABLE '.$db->prefix."bans CHANGE username username VARCHAR(200) DEFAULT NULL") or error('Unable to alter DB structure.', __FILE__, __LINE__, $db->error());
$db->query('ALTER TABLE '.$db->prefix."bans CHANGE ip ip VARCHAR(255) DEFAULT NULL") or error('Unable to alter DB structure.', __FILE__, __LINE__, $db->error());
$db->query('ALTER TABLE '.$db->prefix."bans ADD message VARCHAR(255) DEFAULT NULL AFTER email") or error('Unable to alter DB structure.', __FILE__, __LINE__, $db->error());
$db->query('ALTER TABLE '.$db->prefix."categories CHANGE position disp_position INT(10) NOT NULL DEFAULT '0'") or error('Unable to alter DB structure.', __FILE__, __LINE__, $db->error());
$db->query('ALTER TABLE '.$db->prefix."categories DROP admmod_only") or error('Unable to alter DB structure.', __FILE__, __LINE__, $db->error());
$db->query('ALTER TABLE '.$db->prefix."forums CHANGE last_poster last_poster VARCHAR(200) DEFAULT NULL") or error('Unable to alter DB structure.', __FILE__, __LINE__, $db->error());
$db->query('ALTER TABLE '.$db->prefix."forums CHANGE position disp_position INT(10) NOT NULL DEFAULT '0'") or error('Unable to alter DB structure.', __FILE__, __LINE__, $db->error());
$db->query('ALTER TABLE '.$db->prefix."online CHANGE ident ident VARCHAR(200) NOT NULL") or error('Unable to alter DB structure.', __FILE__, __LINE__, $db->error());
$db->query('ALTER TABLE '.$db->prefix."posts CHANGE poster poster VARCHAR( 200 ) NOT NULL,
											 CHANGE edited_by edited_by VARCHAR(200) DEFAULT NULL") or error('Unable to alter DB structure.', __FILE__, __LINE__, $db->error());
$db->query('ALTER TABLE '.$db->prefix."search_results CHANGE ident ident VARCHAR(200) NOT NULL") or error('Unable to alter DB structure.', __FILE__, __LINE__, $db->error());
$db->query('ALTER TABLE '.$db->prefix."topics CHANGE poster poster VARCHAR(200) NOT NULL,
											  CHANGE subject subject VARCHAR(255) NOT NULL,
											  CHANGE last_poster last_poster VARCHAR(200) DEFAULT NULL,
											  DROP subscribers") or error('Unable to alter DB structure.', __FILE__, __LINE__, $db->error());
$db->query('ALTER TABLE '.$db->prefix."users CHANGE username username VARCHAR(200) NOT NULL") or error('Unable to alter DB structure.', __FILE__, __LINE__, $db->error());
$db->query('ALTER TABLE '.$db->prefix."users CHANGE password password VARCHAR(40) NOT NULL") or error('Unable to alter DB structure.', __FILE__, __LINE__, $db->error());
$db->query('ALTER TABLE '.$db->prefix."users CHANGE hide_email email_setting TINYINT(1) NOT NULL DEFAULT '1'") or error('Unable to alter DB structure.', __FILE__, __LINE__, $db->error());
$db->query('ALTER TABLE '.$db->prefix."users ADD msn VARCHAR(50) DEFAULT NULL AFTER icq") or error('Unable to alter DB structure.', __FILE__, __LINE__, $db->error());
$db->query('ALTER TABLE '.$db->prefix."users ADD show_avatars TINYINT(1) NOT NULL DEFAULT '1' AFTER show_img") or error('Unable to alter DB structure.', __FILE__, __LINE__, $db->error());
$db->query('ALTER TABLE '.$db->prefix."users ADD notify_with_post TINYINT(1) NOT NULL DEFAULT '0' AFTER save_pass") or error('Unable to alter DB structure.', __FILE__, __LINE__, $db->error());
$db->query('ALTER TABLE '.$db->prefix."users ADD last_visit INT(10) UNSIGNED NOT NULL DEFAULT '0' AFTER registered") or error('Unable to alter DB structure.', __FILE__, __LINE__, $db->error());
$db->query('ALTER TABLE '.$db->prefix."users ADD last_action INT(10) UNSIGNED NOT NULL DEFAULT '0' AFTER registered") or error('Unable to alter DB structure.', __FILE__, __LINE__, $db->error());
$db->query('CREATE INDEX '.$db_prefix.'subscriptions_user_id_idx ON '.$db_prefix.'subscriptions(user_id)') or error('Unable to alter DB structure.', __FILE__, __LINE__, $db->error());
$db->query('CREATE INDEX '.$db_prefix.'subscriptions_topic_id_idx ON '.$db_prefix.'subscriptions(topic_id)') or error('Unable to alter DB structure.', __FILE__, __LINE__, $db->error());


// Escape config data
$options = array_map('addslashes', $options);
$permissions = array_map('addslashes', $permissions);

// Insert data into the new config table
$config = array(
	'o_cur_version'				=> "'$update_to'",
	'o_board_title'				=> "'".$options['board_title']."'",
	'o_board_desc'				=> "'".$options['board_desc']."'",
	'o_server_timezone'			=> "'".$options['server_timezone']."'",
	'o_time_format'				=> "'".$options['time_format']."'",
	'o_date_format'				=> "'".$options['date_format']."'",
	'o_timeout_visit'			=> "'".$options['timeout_cookie']."'",
	'o_timeout_online'			=> "'".$options['timeout_online']."'",
	'o_redirect_delay'			=> "'".$options['redirect_delay']."'",
	'o_flood_interval'			=> "'".$options['flood_interval']."'",
	'o_smilies'					=> "'".$options['smilies']."'",
	'o_smilies_sig'				=> "'".$options['smilies_sig']."'",
	'o_make_links'				=> "'".$options['make_links']."'",
	'o_show_post_count'			=> "'".$options['show_post_count']."'",
	'o_default_style'			=> "'".$options['default_style']."'",
	'o_topic_review'			=> "'".$options['topic_review']."'",
	'o_disp_topics_default'		=> "'".$options['disp_topics_default']."'",
	'o_disp_posts_default'		=> "'".$options['disp_posts_default']."'",
	'o_indent_num_spaces'		=> "'".$options['indent_num_spaces']."'",
	'o_quickpost'				=> "'".$options['quickpost']."'",
	'o_users_online'			=> "'".$options['users_online']."'",
	'o_censoring'				=> "'".$options['censoring']."'",
	'o_ranks'					=> "'".$options['ranks']."'",
	'o_show_dot'				=> "'".$options['show_dot']."'",
	'o_quickjump'				=> "'".$options['quickjump']."'",
	'o_gzip'					=> "'".$options['gzip']."'",
	'o_report_method'			=> "'".$options['report_method']."'",
	'o_mailing_list'			=> "'".$options['mailing_list']."'",
	'o_avatars'					=> "'".$options['avatars']."'",
	'o_avatars_dir'				=> "'".$options['avatars_dir']."'",
	'o_avatars_width'			=> "'".$options['avatars_width']."'",
	'o_avatars_height'			=> "'".$options['avatars_height']."'",
	'o_avatars_size'			=> "'".$options['avatars_size']."'",
	'o_search'					=> "'".$options['search']."'",
	'o_search_all_forums'		=> "'".$options['search_all_forums']."'",
	'o_base_url'				=> "'".$options['base_url']."'",
	'o_admin_email'				=> "'".$options['admin_email']."'",
	'o_webmaster_email'			=> "'".$options['webmaster_email']."'",
	'o_subscriptions'			=> "'".$options['subscriptions']."'",
	'o_smtp_host'				=> (($options['smtp_host'] != '') ? "'".$options['smtp_host']."'" : "NULL"),
	'o_smtp_user'				=> (($options['o_smtp_user'] != '') ? "'".$options['o_smtp_user']."'" : "NULL"),
	'o_smtp_pass'				=> (($options['o_smtp_pass'] != '') ? "'".$options['o_smtp_pass']."'" : "NULL"),
	'o_regs_allow'				=> "'".$options['regs_allow']."'",
	'o_regs_verify'				=> "'".$options['regs_validate']."'",
	'o_announcement'			=> "'0'",
	'o_announcement_message'	=> "'Enter your announcement here.'",
	'o_rules'					=> "'".$options['rules']."'",
	'o_rules_message'			=> (($options['rules_message'] != '') ? "'".$options['rules_message']."'" : "'Enter your rules here.'"),
	'o_maintenance'				=> "'".$options['maintenance']."'",
	'o_maintenance_message'		=> "'".$options['maintenance_message']."'",
	'p_guests_read'				=> "'".$permissions['guests_read']."'",
	'p_guests_post'				=> "'".$permissions['guests_post']."'",
	'p_guests_post_topic'		=> "'".$permissions['guests_post_topic']."'",
	'p_guests_search'			=> "'".$permissions['guests_search']."'",
	'p_users_post'				=> "'".$permissions['users_post']."'",
	'p_users_post_topic'		=> "'".$permissions['users_post_topic']."'",
	'p_users_edit_post'			=> "'".$permissions['users_edit_post']."'",
	'p_users_del_post'			=> "'".$permissions['users_del_post']."'",
	'p_users_del_topic'			=> "'".$permissions['users_del_topic']."'",
	'p_users_set_title'			=> "'".$permissions['users_set_title']."'",
	'p_message_bbcode'			=> "'".$permissions['message_bbcode']."'",
	'p_message_img_tag'			=> "'".$permissions['message_img_tag']."'",
	'p_message_all_caps'		=> "'".$permissions['message_all_caps']."'",
	'p_subject_all_caps'		=> "'".$permissions['subject_all_caps']."'",
	'p_sig_all_caps'			=> "'".$permissions['sig_all_caps']."'",
	'p_sig_bbcode'				=> "'".$permissions['sig_bbcode']."'",
	'p_sig_img_tag'				=> "'".$permissions['sig_img_tag']."'",
	'p_sig_length'				=> "'".$permissions['sig_length']."'",
	'p_sig_lines'				=> "'".$permissions['sig_lines']."'",
	'p_allow_banned_email'		=> "'".$permissions['allow_banned_email']."'",
	'p_allow_dupe_email'		=> "'".$permissions['allow_dupe_email']."'"
);

while (list($conf_name, $conf_value) = @each($config))
	$db->query('INSERT INTO '.$db_prefix."config (conf_name, conf_value) VALUES('$conf_name', $conf_value)") or error('Unable to insert config data.', __FILE__, __LINE__, $db->error());


// We've moved last_action and last_visit into the database, so we have to give them values
$now = time();
$db->query('UPDATE '.$db->prefix.'users SET last_visit='.$now.', last_action='.$now) or error('Unable to update user data.', __FILE__, __LINE__, $db->error());


// This feels like a good time to update lastpost/lastposter for all forums
$result = $db->query('SELECT id FROM '.$db->prefix.'forums') or error('Unable to fetch forum info.', __FILE__, __LINE__, $db->error());

while ($row = $db->fetch_row($result))
	update_forum($row[0]);


// We'll empty the search results table as well
$db->query('TRUNCATE TABLE '.$db->prefix.'search_results') or error('Unable to flush search results.', __FILE__, __LINE__, $db->error());


exit('Update successful! Your forum database has now been updated to version '.$update_to.'. You should now remove this script from the forum root directory.');
